import { fixture, html as html$1, elementUpdated, expect } from '@open-wc/testing';
import { LitElement, html, css } from 'lit';
import { property, state } from 'lit/decorators.js';
import { LabelMixin } from '@umbraco-ui/uui-base/lib/mixins';
import { UUIEvent } from '@umbraco-ui/uui-base/lib/events';

class UUIInputEvent extends UUIEvent {
}
UUIInputEvent.CHANGE = "change";

var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __decorateClass = (decorators, target, key, kind) => {
  var result = kind > 1 ? void 0 : kind ? __getOwnPropDesc(target, key) : target;
  for (var i = decorators.length - 1, decorator; i >= 0; i--)
    if (decorator = decorators[i])
      result = (kind ? decorator(target, key, result) : decorator(result)) || result;
  if (kind && result)
    __defProp(target, key, result);
  return result;
};
class UUIInputElement extends LabelMixin("input label", LitElement) {
  constructor() {
    super();
    this.placeholder = "";
    this.disabled = false;
    this.hideLabel = false;
    this._value = "";
    this.name = "";
    this.error = false;
    this.type = "text";
    this._internals = this.attachInternals();
  }
  get value() {
    return this._value;
  }
  set value(newValue) {
    this._value = newValue;
    this._internals.setFormValue(this._value);
  }
  onInput(e) {
    this.value = e.target.value;
  }
  onChange() {
    this.dispatchEvent(new UUIInputEvent(UUIInputEvent.CHANGE, { bubbles: true }));
  }
  render() {
    return html`
      ${this.hideLabel === false ? this.renderLabel() : ""}
      <input
        .type=${this.type}
        .value=${this.value}
        .name=${this.name}
        placeholder=${this.placeholder}
        aria-label=${this.label}
        .disabled=${this.disabled}
        @input=${this.onInput}
        @change=${this.onChange} />
    `;
  }
}
UUIInputElement.styles = [
  css`
      :host {
        display: inline-block;
      }
      input {
        display: inline-block;
        height: 30px;
        padding: 3px 6px 1px 6px;
        font-family: inherit;
        font-size: 15px;
        color: inherit;
        border-radius: 0;
        box-sizing: border-box;
        background-color: var(
          --uui-text-field-background-color,
          var(--uui-interface-surface,#FEFEFE)
        );
        border: 1px solid
          var(--uui-text-field-border-color, var(--uui-interface-border,#C4C4C4));
        width: 100%;
        outline: none;
      }
      input:hover {
        border-color: var(
          --uui-text-field-border-color-hover,
          var(--uui-interface-border-hover,rgb(179, 179, 179))
        );
      }
      input:focus {
        border-color: var(
          --uui-text-field-border-color-focus,
          var(--uui-interface-border-focus,rgb(179, 179, 179))
        );
      }
      :host([invalid]) {
        border-color: var(--uui-color-danger-background);
      }

      :host([type='color']) {
        display: inline-flex;
        align-items: center;
      }

      :host([type='color']) .label {
        margin-left: var(--uui-size-base-unit, 6px);
      }

      input[type='color'] {
        width: 30px;
        padding: 0;
        border: none;
      }

      input[disabled] {
        background-color: var(
          --uui-text-field-background-color-disabled,
          var(--uui-interface-surface-disabled,rgb(228, 228, 228))
        );
        border: 1px solid
          var(
            --uui-text-field-border-color-disabled,
            var(--uui-interface-border-disable)
          );

        color: var(--uui-interface-contrast-disabled,rgb(115, 113, 110));
      }

      :host([disabled]) .label {
        color: var(--uui-interface-contrast-disabled,rgb(115, 113, 110));
      }

      .label {
        font-size: var(--uui-type-small-size, 12px);
        line-height: calc(var(--uui-size-base-unit,6px) * 3);
        font-weight: bold;
      }

      :host([error]) input {
        border: 1px solid var(--uui-look-danger-border, #d42054);
      }

      :host([error]) input[disabled] {
        border: 1px solid var(--uui-look-danger-border, #d42054);
      }
    `
];
UUIInputElement.formAssociated = true;
__decorateClass([
  property()
], UUIInputElement.prototype, "placeholder", 2);
__decorateClass([
  property({ type: Boolean, reflect: true })
], UUIInputElement.prototype, "disabled", 2);
__decorateClass([
  property({ type: Boolean, attribute: "hide-label", reflect: true })
], UUIInputElement.prototype, "hideLabel", 2);
__decorateClass([
  state()
], UUIInputElement.prototype, "_value", 2);
__decorateClass([
  property()
], UUIInputElement.prototype, "value", 1);
__decorateClass([
  property({ type: String })
], UUIInputElement.prototype, "name", 2);
__decorateClass([
  property({ type: Boolean, reflect: true })
], UUIInputElement.prototype, "error", 2);
__decorateClass([
  property({ type: String })
], UUIInputElement.prototype, "type", 2);

customElements.define("uui-input", UUIInputElement);

describe("UuiInput", () => {
  let element;
  let input;
  beforeEach(async () => {
    var _a;
    element = await fixture(html$1` <uui-input label="a input label"></uui-input> `);
    input = (_a = element.shadowRoot) == null ? void 0 : _a.querySelector("input");
  });
  it("test that disable works", async () => {
    element.disabled = true;
    await elementUpdated(element);
    expect(input.disabled).to.be.true;
  });
  it("changes the value to the input value when input event is emitted", async () => {
    input.value = "test value";
    input.dispatchEvent(new Event("input", { bubbles: true, composed: true }));
    expect(element.value).to.equal("test value");
  });
  it("emits a change event when native input fires one", async () => {
    let event = null;
    element.addEventListener("change", (e) => event = e);
    input.dispatchEvent(new Event("change", { bubbles: true, composed: false }));
    expect(event.target).to.equal(element);
  });
});
describe("UuiInput with label", () => {
  let dom;
  beforeEach(async () => {
    dom = await fixture(html$1`
      <label for="test">Input</label>
      <uui-input id="test" label="a input label"></uui-input>
    `);
  });
  it("passes the a11y audit", async () => {
    await expect(dom).shadowDom.to.be.accessible();
  });
});
describe("UuiInput in Form", () => {
  let formElement;
  let element;
  beforeEach(async () => {
    formElement = await fixture(html$1` <form>
        <uui-input
          label="a input label"
          name="input"
          value="Hello uui-input"></uui-input>
      </form>`);
    element = formElement.querySelector("uui-input");
  });
  it("value is correct", async () => {
    await expect(element.value).to.be.equal("Hello uui-input");
  });
  it("form output", async () => {
    const formData = new FormData(formElement);
    await expect(formData.get("input")).to.be.equal("Hello uui-input");
  });
  it("change value and check output", async () => {
    element.value = "anotherValue";
    const formData = new FormData(formElement);
    await expect(formData.get("input")).to.be.equal("anotherValue");
  });
});
